; Bitonal converter v.0.3
;
; change log:
; - image handling. To use this function select the images you want to work on using the mouse 
;	(hold Shift to select more than one image) before using the script
; - changed default opacity to 35
; - changed minimum values of DoG and Gaussians to 1 instead of 0 (which made the script crash)

(define (bitonal-converter image drawable lvlb lvlw lvlg dog1 dog2 gaus1 gaus2 opac thres1 thres2 dither)
	(gimp-image-undo-group-start image)
	(if (or (= dither 4) (= dither 5))
		(let* (
				(drawable (car (gimp-image-get-active-layer image)))
			)	
			(gimp-edit-named-copy drawable "imgcopy")
		)
	)
	(if (not (= dither 0)) (gimp-selection-invert image))
	(if (= (car (gimp-drawable-is-gray drawable)) FALSE)
		(gimp-image-convert-grayscale image)
	)
	(gimp-levels drawable 0 lvlb lvlw lvlg 0 255)
	(let* (
			(dog-layer (car (gimp-layer-copy drawable 0)))
		)
		(gimp-image-add-layer image dog-layer -1)
		(gimp-layer-set-name dog-layer "dog")
		(plug-in-dog RUN-NONINTERACTIVE image dog-layer dog1 dog2 TRUE TRUE)
		(plug-in-gauss RUN-NONINTERACTIVE image dog-layer gaus1 gaus2 1)
		(gimp-layer-set-opacity dog-layer opac)
		(gimp-threshold dog-layer thres1 255)
	)
	(gimp-image-flatten image)
	(let* (
			(drawable (car (gimp-image-get-active-layer image)))
		)
		(gimp-threshold drawable thres2 255)
	)
	(if (not (= dither 0)) (gimp-selection-invert image))
	(cond 
		((equal? dither 1) (gimp-image-convert-indexed image 1 3 1 FALSE TRUE ""))
		((equal? dither 2) (gimp-image-convert-indexed image 1 4 1 FALSE TRUE "Grays"))
		((equal? dither 3) (gimp-image-convert-indexed image 1 0 16 FALSE TRUE ""))
		((equal? dither 4) 
			(let* (
				(drawable (car (gimp-image-get-active-layer image)))
				)
				(gimp-image-convert-rgb image)
				(gimp-edit-named-paste drawable "imgcopy" 0)
			)
			(gimp-buffer-delete "imgcopy")
		)
		((equal? dither 5) 
			(let* (
				(drawable (car (gimp-image-get-active-layer image)))
				)
				(gimp-image-convert-rgb image)
				(gimp-edit-named-paste drawable "imgcopy" 0)
				(gimp-image-convert-indexed image 1 0 256 FALSE TRUE "")
			)
			(gimp-buffer-delete "imgcopy")
		)
	)
	(gimp-displays-flush)
	(gimp-image-undo-group-end image)
)

(script-fu-register "bitonal-converter"
                    "Bitonal converter v.0.3"
                    "Bitonal conversion for DIY scanners using DoG"
                    "Timothee Lecaudey"
                    "Timothee Lecaudey"
                    "2012-05-06"
                    "RGB*, GRAY*"
                    SF-IMAGE "Input Image" 0
                    SF-DRAWABLE "Input drawable" 0
					SF-ADJUSTMENT "Levels Black" '(20 0 255 1 1 0 0)  
					SF-ADJUSTMENT "Levels White" '(220 0 255 1 1 0 0)  
					SF-ADJUSTMENT "Levels Gamma" '(0.67 0 1 0.01 0.01 2 0) 
					SF-ADJUSTMENT "DoG radius 1" '(6 1 10 0.5 0.5 1 0)
					SF-ADJUSTMENT "DoG radius 2" '(1 1 10 0.5 0.5 1 0)
					SF-ADJUSTMENT "Gaussian Blur Horizontal" '(1 1 10 0.1 0.1 1 0)
					SF-ADJUSTMENT "Gaussian Blur Vertical" '(1 1 10 0.1 0.1 1 0)
					SF-ADJUSTMENT "Layer Opacity" '(35 0 100 0.5 0.5 1 0)
					SF-ADJUSTMENT "First threshold" '(240 0 255 0.5 0.5 1 0)
					SF-ADJUSTMENT "Second threshold" '(160 0 255 0.5 0.5 1 0)
					SF-OPTION "Handle images? \n(you need to select them first!)" '("No" "Dither to B&W" "Dither using 'Grays' Palette" "Dither to 16-levels Grayscale" "Color image - copy" "Color image - Dither to 256 colors") 
					)
(script-fu-menu-register "bitonal-converter"
                          "<Image>/Filters/DIY Book Scanning")

(define (bitonal-converter-batch filepath inType outType lvlb lvlw lvlg dog1 dog2 gaus1 gaus2 opac thresh1 thresh2)
	(let* 
		(
			(inExt
				(cond 
					(( equal? inType 0 ) "[tT][iI][fF]" )
					(( equal? inType 1 ) "[jJ][pP][gG]" )
					(( equal? inType 2 ) "[bB][mM][pP]" )
					(( equal? inType 3 ) "[pP][nN][gG]" )
				)
			)
			(filelist (cadr (file-glob (string-append filepath DIR-SEPARATOR "*." inExt) 1)))
		)
		(while (not (null? filelist))
			(let* 
				(
					(filename (car filelist))
					(image (car (gimp-file-load RUN-NONINTERACTIVE filename filename)))
					(drawable (car (gimp-image-get-active-layer image)))
				)
				(bitonal-converter image drawable lvlb lvlw lvlg dog1 dog2 gaus1 gaus2 opac thresh1 thresh2)
				(let* 
					(
						(drawable (car (gimp-image-get-active-layer image)))
					)
					(cond 
						((equal? outType 0 )
							(file-tiff-save RUN-NONINTERACTIVE 
								image drawable 	(string-append filepath DIR-SEPARATOR "_" (substring filename (+ (string-length filepath) 1) (- (string-length filename) 4)) ".tif") 
												(string-append filepath DIR-SEPARATOR "_" (substring filename (+ (string-length filepath) 1) (- (string-length filename) 4)) ".tif") 1
							)
						)
						((equal? outType 1 )
							(file-png-save-defaults RUN-NONINTERACTIVE 
								image drawable 	(string-append filepath DIR-SEPARATOR "_" (substring filename (+ (string-length filepath) 1) (- (string-length filename) 4)) ".png") 
												(string-append filepath DIR-SEPARATOR "_" (substring filename (+ (string-length filepath) 1) (- (string-length filename) 4)) ".png")
							)
						)
						((equal? outType 2 )
							(file-gif-save RUN-NONINTERACTIVE 
								image drawable 	(string-append filepath DIR-SEPARATOR "_" (substring filename (+ (string-length filepath) 1) (- (string-length filename) 4)) ".gif") 
												(string-append filepath DIR-SEPARATOR "_" (substring filename (+ (string-length filepath) 1) (- (string-length filename) 4)) ".gif") 0 0 0 0
							)
						)
						((equal? outType 3 )
							(file-pbm-save RUN-NONINTERACTIVE 
								image drawable 	(string-append filepath DIR-SEPARATOR "_" (substring filename (+ (string-length filepath) 1) (- (string-length filename) 4)) ".pbm") 
												(string-append filepath DIR-SEPARATOR "_" (substring filename (+ (string-length filepath) 1) (- (string-length filename) 4)) ".pbm") 1
							)
						)
					)
				)
				(gimp-image-delete image)
			)
			(set! filelist (cdr filelist))
		)
	)
)
(script-fu-register "bitonal-converter-batch"
                    "Batch Bitonal converter v.0.3"
                    "Batch Bitonal conversion for DIY scanners using DoG"
                    "Timothee Lecaudey"
                    "Timothee Lecaudey"
                    "2012-05-07"
                    "RGB*, GRAY*"
					SF-DIRNAME "Input Folder" "C:\\"
					SF-OPTION "Input Type" (list ".tif" ".jpg" ".bmp" ".png") 
					SF-OPTION "Output Type" (list ".tif" ".png" ".gif" ".pbm")
					SF-ADJUSTMENT "Levels Black" '(20 0 255 1 1 0 0)  
					SF-ADJUSTMENT "Levels White" '(220 0 255 1 1 0 0)  
					SF-ADJUSTMENT "Levels Gamma" '(0.67 0 1 0.01 0.01 2 0) 
					SF-ADJUSTMENT "DoG radius 1" '(6 1 10 0.5 0.5 1 0)
					SF-ADJUSTMENT "DoG radius 2" '(1 1 10 0.5 0.5 1 0)
					SF-ADJUSTMENT "Gaussian Blur Horizontal" '(1 1 10 0.1 0.1 1 0)
					SF-ADJUSTMENT "Gaussian Blur Vertical" '(1 1 10 0.1 0.1 1 0)
					SF-ADJUSTMENT "Layer Opacity" '(35 0 100 0.5 0.5 1 0)
					SF-ADJUSTMENT "First threshold" '(240 0 255 0.5 0.5 1 0)
					SF-ADJUSTMENT "Second threshold" '(160 0 255 0.5 0.5 1 0)
					)
(script-fu-menu-register "bitonal-converter-batch"
                          "<Image>/Filters/DIY Book Scanning")